using BepInEx;
using BepInEx.Configuration;
using R2API;
using R2API.Utils;
using RoR2;
using RoR2.Skills;
using System;
using UnityEngine;

namespace ArtiTweaks
{
    [BepInDependency(R2API.R2API.PluginGUID)]
    [BepInPlugin(PluginGUID, PluginName, PluginVersion)]
	
    public class ArtiTweaks : BaseUnityPlugin
	{
        public const string PluginGUID = PluginAuthor + "." + PluginName;
        public const string PluginAuthor = "Foldex";
        public const string PluginName = "ArtiTweaks";
        public const string PluginVersion = "1.0.0";

        public static bool AntiGravEnabled = false;
        public static float AntiGravModifier = 1f;

        public static bool FlamethrowerRangeEnabled = true;
        public static float FlamethrowerRangeModifier = 1.7f;

        public static bool IonSurgeUtilityEnabled = true;

        public static bool SprintingFlamethrowerEnabled = false;
        public static bool SprintingM2Enabled = true;

        public void Awake()
        {

            LoadConfig();

            // AntiGravityForce
            if (AntiGravEnabled)
            {
                On.RoR2.AntiGravityForce.FixedUpdate += (orig, self) =>
                {
                    self.antiGravityCoefficient = AntiGravModifier;
                    orig(self);
                };
            }

            // Flamethrower
            if (FlamethrowerRangeEnabled)
            {
                On.EntityStates.Mage.Weapon.Flamethrower.OnEnter += (orig, self) =>
                {
                    self.maxDistance *= FlamethrowerRangeModifier;
                    orig(self);
                };
            }

            // Skill Defs
            On.RoR2.Skills.SkillCatalog.Init += (orig) =>
            {
                orig();

                if (SprintingFlamethrowerEnabled)
                {
                    SkillDef flamethrower = Resources.Load<SkillDef>("skilldefs/magebody/MageBodyFlamethrower");
                    if (flamethrower)
                    {
                        flamethrower.canceledFromSprinting = false;
                        flamethrower.cancelSprintingOnActivation = false;
                    }
                }

                if (SprintingM2Enabled)
                {
                    SkillDef novabomb = Resources.Load<SkillDef>("skilldefs/magebody/MageBodyNovaBomb");
                    if (novabomb)
                    {
                        novabomb.cancelSprintingOnActivation = false;
                    }

                    SkillDef icespear = Resources.Load<SkillDef>("skilldefs/magebody/MageBodyIceBomb");
                    if (icespear)
                    {
                        icespear.cancelSprintingOnActivation = false;
                    }

                }

                if (IonSurgeUtilityEnabled)
                {
                    GameObject Arti = Resources.Load<GameObject>("prefabs/characterbodies/MageBody");
                    var IonSurge = Resources.Load<SkillDef>("skilldefs/magebody/MageBodyFlyUp");
                    var skillLocator = Arti.GetComponent<SkillLocator>();
                    var skillFamily = skillLocator.utility.skillFamily;

                    Array.Resize(ref skillFamily.variants, skillFamily.variants.Length + 1);
                    skillFamily.variants[skillFamily.variants.Length - 1] = new SkillFamily.Variant
                    {
                        skillDef = IonSurge,
                        unlockableDef = null,
                        viewableNode = new ViewablesCatalog.Node(IonSurge.skillNameToken, false, null)
                    };
                }
                    
            };
            
        }

        private void LoadConfig()
        {

            AntiGravEnabled = base.Config.Bind<bool>(new ConfigDefinition("Gravity", "Anti Gravity Enabled"), false, new ConfigDescription("Enable modifying Anti Gravity")).Value;
            AntiGravModifier = base.Config.Bind<float>(new ConfigDefinition("Gravity", "Anti Gravity Value"), 1f, new ConfigDescription("Affects Gravity on Artificer's NanoBomb and Void Fiend's Corrupted Flood. No Gravity: 1, Vanilla Value: 0.7")).Value; 

            FlamethrowerRangeEnabled = base.Config.Bind<bool>(new ConfigDefinition("Flamethrower", "Range Enabled"), true, new ConfigDescription("Enable modifying Flamethrower range")).Value;
            FlamethrowerRangeModifier = base.Config.Bind<float>(new ConfigDefinition("Flamethrower", "Range Modifier"), 1.7f, new ConfigDescription("Multiply Flamethrower range by this value. Max: 3.0")).Value;
            
            if (FlamethrowerRangeModifier > 3.0) {
                FlamethrowerRangeModifier = 3.0f;
            }

            IonSurgeUtilityEnabled = base.Config.Bind<bool>(new ConfigDefinition("Ion Surge", "Utility Enabled"), true, new ConfigDescription("Enable adding Ion Surge to the Utility Slot")).Value;
            
            SprintingFlamethrowerEnabled = base.Config.Bind<bool>(new ConfigDefinition("Sprinting", "Flamethrower Enabled"), false, new ConfigDescription("Enable sprinting during Flamethrower usage")).Value;
            SprintingM2Enabled = base.Config.Bind<bool>(new ConfigDefinition("Sprinting", "M2 Enabled"), true, new ConfigDescription("Enable sprinting during M2 usage")).Value;

        }

    }
}
